#!/usr/bin/perl -w
use strict;
use Getopt::Long;
use File::Basename;
use List::Util qw[min max];


###############################################################
##################### DOCUMENTATION ###########################

## Author: Yun Li
my $location = $0;
my $title = "Weighted-Haplotype Test for Multiple Rare Variants (Wrapper): Version 0.0.1";

print "$title\n";
print "\t\@: $location\n";
print "\tby: Yun Li(ylwtx\@med.unc.edu)\n\n";

## Input Parameters
##

## Input Files
##

## Output Files
################# END OF DOCUMENTATION ########################
###############################################################
my $starttime = time();

my $MIN_NUM_OPTS = 2;

if($#ARGV<($MIN_NUM_OPTS*2 - 1))
{
    &usage();
    die "Failed to parse parameters\n\n";
}

my %opts = ();

    # Default Optional Options
$opts{tr} = 0.3;
$opts{o} = "wh.out";
$opts{rareL} = 0;
$opts{rareU} = 0.5;

Getopt::Long::GetOptions(\%opts,qw(
    cth=s
    csh=s
    tr=f
    o=s
    rareL=f
    rareU=f
)) || die "Failed to parse options\n\n";



&main();

my $endtime = time();
my $lapsetime = $endtime - $starttime;
print "\nAnalysis took $lapsetime seconds\n\n";


# begin sub

sub usage
{
    print "\n";
    print "Usage:\n\t";
    print "-cth\t Control Haplotypes (Required) \n\t";
    print "-csh\t Case Haplotypes (Required) \n\t";
    print "-rareL\t Lower Bound of MAF (Optional, default = 0) \n\t";
    print "-rareU\t Upper Bound of MAF (Optional, default = 0.5) \n\t";
    print "-tr\t proportion used for training (Optional, default = 0.3) \n\t";
    print "-o\t Output Prefix (Optional) \n\t";
    print "\n";
}

sub main
{
  print("haps2risky.csh $opts{csh},$opts{cth},1,$opts{o}.risky,$opts{tr}\n");
  system("haps2risky.csh $opts{csh},$opts{cth},1,$opts{o}.risky,$opts{tr}");
  print("haps2protective.csh $opts{csh},$opts{cth},1,$opts{o}.protective,$opts{tr}\n");
  system("haps2protective.csh $opts{csh},$opts{cth},1,$opts{o}.protective,$opts{tr}");
  system("rm -f $opts{o}.diff.training.hap");
  system("touch $opts{o}.diff.training.hap");
  system("sed -e 's/\$/ +/g' $opts{o}.risky.training.rr.hap > $opts{o}.diff.training.hap");
  system("sed -e 's/\$/ -/g' $opts{o}.protective.training.rp.hap >> $opts{o}.diff.training.hap");
  system("rm -f $opts{o}.risky.training.rr.hap $opts{o}.protective.training.rp.hap");
  my $numberDiffHap = `wc -l < $opts{o}.diff.training.hap`;
  chomp($numberDiffHap);
  if ($numberDiffHap == 0)
  {
    system("haps2score.pl -rareL $opts{rareL} -rareU $opts{rareU} -csh $opts{csh} -cth $opts{cth} -col 1 -o $opts{o}.whs.diff.rare.score");
  }
  else
  {
    system("haps2score.pl -rareL $opts{rareL} -rareU $opts{rareU} -risky $opts{o}.diff.training.hap -csh $opts{csh} -cth $opts{cth} -col 1 -o $opts{o}.whs.diff.rare.score");
  }
  system("rm -f $opts{o}.diff.training.hap");
  
  my $cshap = `wc -l < $opts{csh}`;
  my $cthap = `wc -l < $opts{cth}`;
  chomp($cshap);
  chomp($cthap);
  my $testcases    = int(($cshap * (1.0 - $opts{tr})) / 2.0);
  my $testcontrols = int(($cthap * (1.0 - $opts{tr})) / 2.0);
  system("head -1 $opts{o}.whs.diff.rare.score > $opts{o}.whs.diff.rare.testing.txt");
  system("awk '\$2 == 1' $opts{o}.whs.diff.rare.score | tail -n $testcontrols >> $opts{o}.whs.diff.rare.testing.txt");
  system("awk '\$2 == 2' $opts{o}.whs.diff.rare.score | tail -n $testcases    >> $opts{o}.whs.diff.rare.testing.txt");
  system("rm -f $opts{o}.whs.diff.rare.score");
  my $pval = `cat wilcox.greater.r | R --vanilla --args $opts{o}.whs.diff.rare.testing.txt | grep p-value | awk '{print \$6}'`;
  print("\n\nWeighted Haplotype p-value is $pval\n\n");
  system("rm -f $opts{o}.whs.diff.rare.testing.txt");
}

