#!/usr/bin/perl -w
use strict;
use Getopt::Long;
use File::Basename;
use List::Util qw[min max];


###############################################################
##################### DOCUMENTATION ###########################

## Author: Yun Li
my $location = $0;
my $title = "Weighted Imputation Dosage Test for Multiple Rare Variants (Wrapper): Version 0.0.1";

print "$title\n";
print "\t\@: $location\n";
print "\tby: Yun Li(ylwtx\@med.unc.edu)\n\n";

## Input Parameters
##

## Input Files
##

## Output Files
################# END OF DOCUMENTATION ########################
###############################################################
my $starttime = time();

my $MIN_NUM_OPTS = 3;

if($#ARGV<($MIN_NUM_OPTS*2 - 1))
{
    &usage();
    die "Failed to parse parameters\n\n";
}

my %opts = ();

    # Default Optional Options
$opts{tr} = 0.3;
$opts{risky} = 1;
$opts{o} = "wid.out";
$opts{rareL} = 0;
$opts{rareU} = 0.5;

Getopt::Long::GetOptions(\%opts,qw(
    ctd=s
    csd=s
    info=s
    tr=f
    risky=i
    o=s
    rareL=f
    rareU=f
)) || die "Failed to parse options\n\n";



&main();

my $endtime = time();
my $lapsetime = $endtime - $starttime;
print "\nAnalysis took $lapsetime seconds\n\n";


# begin sub

sub usage
{
    print "\n";
    print "Usage:\n\t";
    print "-ctd\t Control Dosages (Required) \n\t";
    print "-csd\t Case Dosages (Required) \n\t";
    print "-info\t Marker Information File (Required) \n\t";
    print "-rareL\t Lower Bound of MAF (Optional, default = 0) \n\t";
    print "-rareU\t Upper Bound of MAF (Optional, default = 0.5) \n\t";
    print "-tr\t proportion used for training (Optional, default = 0.3) \n\t";
    print "-risky\t  (Optional) \n\t";
    print "-o\t Output Prefix (Optional) \n\t";
    print "\n";
}

sub main
{
  print("dosages2scores.pl -csd $opts{csd} -ctd $opts{ctd} -info $opts{info} -rareL $opts{rareL} -rareU $opts{rareU} -risky $opts{risky} -tr $opts{tr} -o $opts{o}.dosage.score.txt");
  system("dosages2scores.pl -csd $opts{csd} -ctd $opts{ctd} -info $opts{info} -rareL $opts{rareL} -rareU $opts{rareU} -risky $opts{risky} -tr $opts{tr} -o $opts{o}.dosage.score.txt");
  
  my $csdose = `wc -l < $opts{csd}`;
  my $ctdose = `wc -l < $opts{ctd}`;
  chomp($csdose);
  chomp($ctdose);
  my $testcases    = int(($csdose * (1.0 - $opts{tr})));
  my $testcontrols = int(($ctdose * (1.0 - $opts{tr})));
  if ($opts{tr} == 1.0)
  {
    print "WANING: all used for training AND testing. Permutation is needed to ensure the correct Type-I error\n";
    $testcases    = $csdose;
    $testcontrols = $ctdose;
  }
  system("wc -l $opts{o}.dosage.score.txt");
  system("rm -f $opts{o}.dosage.testing.score.txt");
  system("head -1 $opts{o}.dosage.score.txt > $opts{o}.dosage.testing.score.txt"); 
  system("awk '\$2 == 1' $opts{o}.dosage.score.txt | tail -n $testcontrols >> $opts{o}.dosage.testing.score.txt");
  system("awk '\$2 == 2' $opts{o}.dosage.score.txt | tail -n $testcases    >> $opts{o}.dosage.testing.score.txt");
  system("rm -f $opts{o}.dosage.score.txt");
  my $pval = `cat wilcox.greater.r | R --vanilla --args $opts{o}.dosage.testing.score.txt | grep p-value | awk '{print \$6}'`;
  print("\n\nWeighted Imputation Dosage p-value is $pval\n\n");
  system("rm -f $opts{o}.dosage.testing.score.txt");
}

