#include "StringHash.h"
#include "StringArray.h"
#include "InputFile.h"

#include <string.h>

int main(int argc, char ** argv)
   {
   StringArray    header;
   StringIntHash  entries;
   StringHash     text;

   String         input;
   StringArray    buffer;
   String         missing = "MISS";

   bool outer_join = false;
   if (argc > 2 && strcmp(argv[1], "--outer-join") == 0)
     {
     outer_join = true;
     argc--;
     argv++;
     }

   if (argc < 2)
      {
      printf("This program will merge a series of tables according to a primary key in the first column.\n\n"
             "Usage: mergeTables [--outer-join] table1 table2 ...\n");
      exit(-1);
      }

   for (int i = 1; i < argc; i++)
      {
      IFILE f = ifopen(argv[i], "rb");

      if (f == NULL)
         {
         printf("Error opening file '%s'\n", (const char *) argv[i]);
         return 0;
         }

      buffer.ReplaceTokens(input.ReadLine(f));

      for (int j = 0; j < buffer.Length(); j++)
         if (header.SlowFind(buffer[j]) >= 0)
             header.Push(buffer[j] + "." + i);
         else 
             header.Push(buffer[j]);

      int columns = buffer.Length();

      while (!ifeof(f))
         {
         buffer.ReplaceTokens(input.ReadLine(f));

         if (buffer.Length() == 0) continue;

         if (entries.GetCount(buffer[0]) < i - 1) continue;

         entries.IncrementCount(buffer[0]);

         if (i == 1)
            text.SetObject(buffer[0], new String);

         String * items = (String *) text.Object(buffer[0]);

         if (items == NULL)
            continue;

         if (i != 1)
            *items += '\t';

         *items += input;
         }

      // String * test = (String *) text.Object("rs12286037");
      //
      // if (test != NULL)
      //   printf("Test[%d]: %s\n", entries.Integer("rs12286037")
      //                          , (const char *) * (String *) test );

      ifclose(f);

      if (!outer_join) continue;

      String padding;
      for (int j = 0; j < columns; j++)
         padding += "\t"+missing;

      for (int j = 0; j < text.Capacity(); j++)
         if (text.SlotInUse(j) && (entries.GetCount(text[j]) != i))
            {
            entries.IncrementCount(text[j]);
            * (String *) text.Object(j) += padding;
            }
      }

   for (int i = 0; i < header.Length() - 1; i++)
      printf("%s\t", (const char *) header[i]);
   printf("%s\n", (const char *) header.Last());

   for (int i = 0; i < text.Capacity(); i++)
      if (text.SlotInUse(i) && (entries.Integer(text[i]) == argc - 1))
         printf("%s\n", (const char *) * (String *) text.Object(i));
   }
